/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include "Winbase.h"
#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>

#include "base/startcmd.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/SettingFunctions.h"
#include "client/Sync4jClient.h"
#include "pim/ContactSyncSource.h"
#include "pim/CalendarSyncSource.h"
#include "pim/NoteSyncSource.h"
#include "pim/TaskSyncSource.h"
#include "pim/BriefcaseSyncSource.h"

#include "pim/mail/MailSyncSource.h"
#include "client/MailSourceManagementNode.h"
#include "filter/SourceFilter.h"
#include "filter/ClauseUtil.h"

#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "client/SourceManagementNode.h"
#include "syncml/formatter/Formatter.h"

#include "Notify.h"
#include "notify/timed_msgbox.h"

#include "pim/difftime.h"

int checkRefreshFromServer(int type);
#define OL_MAIL 610
/*      
 * Entry point from VB application. 
 * Accept and manage Variant array of String, long and int. 
 * The entry array is a String array in which elements rapresent SyncSource name.
 * Now accepted and managed element names are "contacts" and "appointments"
 * Array contains at least one element
 * @param vbErrorMsg   : the variable to store the message to return to vb GUI interface
 * @param path       : the install path of the SyncClient PIM application on the device
 */

DWORD WINAPI synchronization (wchar_t* vbErrorMsg, wchar_t* path) {
    
    wchar_t CONTACTS    [15];
    wchar_t APPOINTMENTS[15];       
    wchar_t TASKS       [15];
    wchar_t FAVORITES   [15];
    wchar_t FILES       [15];
    wchar_t NOTES       [15];
    wchar_t MAILS       [15];    
    wchar_t errorMessage[1024];
    int rc          = 0;
    lastErrorCode   = 0;
    ArrayList syncSources;
    char* value;
    errorMessage[0] = 0;
    SyncSource** ssArray = NULL;
    int ret = 0;
    BOOL isMailToSync = FALSE;
    int mailSourceIndex = -1; 
    int result = 0;
    wsprintf(CONTACTS,      CONTACTS_NAME    );
    wsprintf(APPOINTMENTS,  APPOINTMENTS_NAME);
    wsprintf(TASKS,         TASKS_NAME       );
    wsprintf(FAVORITES,     FAVORITES_NAME   );
    wsprintf(FILES,         FILES_NAME       );
    wsprintf(NOTES,         NOTES_NAME       );
    wsprintf(MAILS,         MAILS_NAME       );

    //LOG.setLogPath(path);
    
    // to remember code only...
    // int constFolderCalendar  =  9; 
    // int constFolderContacts  = 10;
    // int constFolderTasks     = 13;

    //
    // custom values
    //
    // int constFiles       = 600;
    // int constNotes       = 601;
    // int constFavorites   = 602; 
    
    LogLevel ll = getLOGLevel();
    LOG.setLevel(ll);        

    // 
    // The list of the sync sources to sync
    //
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);		
	ManagementNode* node = NULL;
    

    // contact
    node = dmt->getManagementNode(CONTACTS_CONTEXT);
	value = node->getPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
                        
        ContactSyncSource contactSource = ContactSyncSource(CONTACTS, OL_CONTACTS);             
        contactSource.setPath(path);
        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_CONTACTS);
        }        
        if (result == 0)
            syncSources.add(contactSource);

    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    // calendar
    node = dmt->getManagementNode(APPOINTMENTS_CONTEXT);	
	value = node->getPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
        CalendarSyncSource  s  = CalendarSyncSource (APPOINTMENTS,     OL_CALENDAR);         
        s.setPath(path);
        
        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_CALENDAR);
        }        
        if (result == 0)            
            syncSources.add(s);
    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    // task
    node = dmt->getManagementNode(TASKS_CONTEXT);
	value = node->getPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
        TaskSyncSource  s  = TaskSyncSource (TASKS,     OL_TASK);         
        s.setPath(path);

        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_TASK);
        }        
        if (result == 0)            
            syncSources.add(s);        
    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
     // briefcase
    node = dmt->getManagementNode(BRIEFCASE_CONTEXT);
	value = node->getPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
        BriefcaseSyncSource  s  = BriefcaseSyncSource (FILES,     OL_BRIEFCASE);         
        s.setPath(path);
        //if (node) { delete node ; node = NULL; }
        //node = dmt->getManagementNode(BRIEFCASE_CONTEXT);        
        wchar_t *dir = toWideChar(node->getPropertyValue("dir"));
        s.setDir(dir);
        
        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_BRIEFCASE);
        }        
        if (result == 0)            
            syncSources.add(s);       

        if (dir) { delete [] dir; dir = NULL; }
    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
     // note
    node = dmt->getManagementNode(NOTES_CONTEXT);
	value = node->getPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
        NoteSyncSource  s  = NoteSyncSource (NOTES,     OL_NOTES);         
        s.setPath(path);
        //if (node) { delete node ; node = NULL; }
        //node = dmt->getManagementNode(NOTES_CONTEXT);
        wchar_t *dir = toWideChar(node->getPropertyValue("dir"));
        s.setDir(dir);

        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_NOTES);
        }        
        if (result == 0)            
            syncSources.add(s);      

        if (dir) { delete [] dir; dir = NULL; }
    }
    if (dmt) { delete dmt; dmt = NULL; }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    /*
    // mail
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");   
    MailSyncSourceConfig sc;    
    wchar_t t[12];
    sc = m.getMailSourceConfig(true);    
    MailSyncSource mailSource = MailSyncSource (MAILS); 
    
    if (strcmp(sc.getSync(), "none") != 0) {
        getFolderToSync(sc, t);    
    
        int days = sc.getDownloadAge();
        wchar_t* downloadAge = getDayBefore(days);

        SourceFilter* filter = 
            ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(),sc.getAttachSize());        

        mailSource.setPath(path);
        mailSource.setFolderToSync(t);
        mailSource.setFilter(filter);

        syncSources.add(mailSource);
        isMailToSync = TRUE;
        mailSourceIndex = syncSources.size()-1;        

    }   
    */
    if (syncSources.size() == 0) {
        wsprintf(errorMessage, TEXT("No sources to sync"));
        LOG.debug("%s", ("No PIM sources to sync"));
        ret = -10;
    
    } else {
        
        Sync4jClient& s4j = Sync4jClient::getInstance();
        s4j.setDMConfig(APPLICATION_URI);   
    
        //ret = s4j.sync(syncSources);         
        
        int l = syncSources.size();
        const int ml = 10;        
        SyncSource* array[ml];    
        for (int i=0; (i<l) && (i<ml-1); ++i) {
            array[i] = (SyncSource*)syncSources[i];
        }
        array[l] = NULL;
        ret = s4j.sync(array);
        
        /*
        if (ret == 0) {
        // If the mail has been synchronized
            if (mailSourceIndex >= 0) {
                MailSyncSource *s = (MailSyncSource *)array[mailSourceIndex];

                // Re-sync if a message was in outbox, to get the result from the server
                if ( s->getIsMailInOutbox() ) {
                    // 1) If the mail source is in a position different from 0, copy
                    //    it to the first position
                    if(mailSourceIndex > 0)
                        array[0] = s;
                    // 2) Truncate the array
                    array[1] = NULL;
                    // 3) Do the second sync for mail source
                    s4j.setDMConfig(APPLICATION_URI);
                    ret = s4j.sync(array);
                }
                // Alert user if there is new mail
                if (s->getIsMailInInbox()) {
                    MessageBeep(MB_ICONASTERISK);
                }
                if (s->getFailedSendMailInOutbox()) {
                    TimedMessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
                        MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, OUT_MESSAGE_TIMEOUT*1000);
                }
            }
        }
        //
        // If there is a mail in Outbox folder, the sync is performed twice.
        // 
       */
        Sync4jClient::dispose();
    }
    
    //
    // NOTE: when passing a string from VB to a DLL, it looks like
    // VB does not want that the last chararcter in the string is
    // the terminator. Therefore we change it to a space. However,
    // IT IS VERY IMPORTANT that the string allocated by the caller
    // less than the length of errorMessage. 
    //
    wsprintf(vbErrorMsg, errorMessage);
    vbErrorMsg[wcslen(vbErrorMsg)] = L' ';
    vbErrorMsg[wcslen(vbErrorMsg)+1] = 0;    
    LOG.info(T("Sync ended."));
    
    return ret;

}

/*      
 * Entry point to kickoff the synchronization process, specifying which
 * sources to synchronize.
 *
 * @param path    : the install path of the SyncClient PIM application on 
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 */

DWORD WINAPI synchronizeMultipleSources (const wchar_t* path, const wchar_t** sources) {
    
    if (sources == 0) {
        return 0;
    }

    int ret = 0;
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);	
	ManagementNode* node = NULL;
    int i=0;
    ArrayList syncSources;
    int mailSourceIndex = -1;
    int result = 0;
    char* value = NULL;

    LogLevel ll = getLOGLevel();
    LOG.setLevel(ll);
    //LOG.setLogPath(path);

    for (i=0; sources[i]; i++) {
        if (wcscmp(sources[i], CONTACTS_NAME) == 0) {
            ContactSyncSource s(CONTACTS_NAME, OL_CONTACTS);
            s.setPath(path);

            node = dmt->getManagementNode(CONTACTS_CONTEXT);
	        value = node->getPropertyValue("sync");
            
            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_CONTACTS);
            }        
            if (result == 0)
                syncSources.add(s);
            
            if (node) { delete node; node = NULL; }
            if (value) { delete [] value; value = NULL; }

        }
        else if (wcscmp(sources[i], APPOINTMENTS_NAME) == 0) {
            CalendarSyncSource s(APPOINTMENTS_NAME, OL_CALENDAR);
            s.setPath(path);
            
            node = dmt->getManagementNode(APPOINTMENTS_CONTEXT);
	        value = node->getPropertyValue("sync");

            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_CALENDAR);
            }        
            if (result == 0)
                syncSources.add(s);

            if (node) { delete node; node = NULL; }
            if (value) { delete [] value; value = NULL; }
        }
        else if (wcscmp(sources[i], TASKS_NAME)        == 0) {
            TaskSyncSource  s(TASKS_NAME, OL_TASK);         
            s.setPath(path);
            
            node = dmt->getManagementNode(TASKS_CONTEXT);
	        value = node->getPropertyValue("sync");

            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_TASK);
            }        
            if (result == 0)
                syncSources.add(s);

            if (node) { delete node; node = NULL; }
            if (value) { delete [] value; value = NULL; }
        }
        else if (wcscmp(sources[i], FAVORITES_NAME)    == 0) {
/*
            FavoritesSyncSource s(FAVORITES_NAME, OL_FAVORITES);
            s.setPath(path);
            syncSources.add(s);
*/
        }
        else if (wcscmp(sources[i], FILES_NAME)        == 0) {   
            BriefcaseSyncSource  s(FILES_NAME, OL_BRIEFCASE);         
            s.setPath((wchar_t*)path);
            
            node = dmt->getManagementNode(BRIEFCASE_CONTEXT);
            if (node) {
                wchar_t* dir = toWideChar(node->getPropertyValue("dir"));
                s.setDir(dir);
                               
    	        value = node->getPropertyValue("sync");

                result = 0;
                if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                    result = checkRefreshFromServer(OL_BRIEFCASE);
                }        
                if (result == 0)
                    syncSources.add(s);
                
                if (node) { delete node; node = NULL; }
                if (value) { delete [] value; value = NULL; }
                if (dir) { delete [] dir; dir = NULL; }
            }
            else {
                LOG.error(T("Can't access management node %s"), BRIEFCASE_CONTEXT);
            }
        } 
        else if (wcscmp(sources[i], NOTES_NAME) == 0) {
            NoteSyncSource s(NOTES_NAME, OL_NOTES);         
            s.setPath(path);
            
            node = dmt->getManagementNode(NOTES_CONTEXT);
            if (node) {
                wchar_t* dir = toWideChar(node->getPropertyValue("dir"));
                s.setDir(dir);
                
                value = node->getPropertyValue("sync");

                result = 0;
                if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                    result = checkRefreshFromServer(OL_NOTES);
                }        
                if (result == 0)
                    syncSources.add(s);

                if (node) { delete node; node = NULL; }
                if (value) { delete [] value; value = NULL; }
                if (dir) { delete [] dir; dir = NULL; }
            } 
            else {
                LOG.error(T("Can't access management node %s"), NOTES_CONTEXT);
            }
        }
        /*
        else if (wcscmp(sources[i], MAILS_NAME) == 0) {
            MailSourceManagementNode m(SOURCES_CONTEXT, "mails");   
            MailSyncSourceConfig sc;    
            wchar_t t[12];
            sc = m.getMailSourceConfig(true);    
       
            getFolderToSync(sc, t);    
    
            int days = sc.getDownloadAge();
            wchar_t* downloadAge = getDayBefore(days);

            SourceFilter* filter = 
                ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(),sc.getAttachSize());

            MailSyncSource mailSource(MAILS_NAME);
            
            mailSource.setPath(path);
            mailSource.setFolderToSync(t);
            mailSource.setFilter(filter);

            syncSources.add(mailSource);

            mailSourceIndex = syncSources.size()-1;
        }
        */
    }
    
    if (syncSources.size() == 0) {
       LOG.debug("%s", ("No PIM sources to sync"));
       ret = -10;
    
    } else {
        //
        // Now we can create the Sync4jClient object and kick off the sync
        //
        Sync4jClient& s4j = Sync4jClient::getInstance();
        s4j.setDMConfig(APPLICATION_URI);
       

        //ret = s4j.sync(syncSources); 
        int l = syncSources.size();
        const int ml = 10;        
        SyncSource* s_array[ml];
        for (i=0; (i<l) && (i<ml-1); ++i) {
            s_array[i] = (SyncSource*)syncSources[i];
        }
        s_array[i] = NULL;
        ret = s4j.sync(s_array);
        /*
        if (ret == 0) {
            // If the mail has been synchronized
            if (mailSourceIndex >= 0) {
                MailSyncSource *s = (MailSyncSource *)s_array[mailSourceIndex];

                // Re-sync if a message was in outbox, to get the result from the server
                if ( s->getIsMailInOutbox() ) {
                    // 1) If the mail source is in a position different from 0, copy
                    //    it to the first position
                    if(mailSourceIndex > 0)
                        s_array[0] = s;
                    // 2) Truncate the array
                    s_array[1] = NULL;
                    // 3) Do the second sync for mail source
                    s4j.setDMConfig(APPLICATION_URI);
                    ret = s4j.sync(s_array);
                }
                // Alert user if there is new mail
                if (s->getIsMailInInbox()) {
                    MessageBeep(MB_ICONASTERISK);
                }
                if (s->getFailedSendMailInOutbox()) {
                    TimedMessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
                        MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, OUT_MESSAGE_TIMEOUT*1000);
                }
            }
        }
        */
        Sync4jClient::dispose();
        LOG.debug(T("Sync ended multiple sources."));
    }
    return ret;
}

/**
 * Return TRUE if the process is started
 */
BOOL startPolling()
{    
    return (startcmd(SYNCAPP, APP_RUN_AT_TIME) > 0);
}

/*
 * Return 0: process terminated successfully
 *        >0: error code from child process    
 *       -1: no such process/invalid handle
 *       -2: timeout
 *       -3: can't get child exit code
 *       -5: can't run the app
 *       -6: the application is stopped by user (used into startSyncPid and checkPid)
 */

int startSync(time_t timeout)
{
    if (!timeout)
        timeout=5*60; // default timeout 5 minutes

    unsigned long pid = startcmd(SYNCAPP, TEXT("once"));
   
    if (pid) {
        return waitProcess(pid, timeout*1000);        
    }
    else {
        return -5;        
    }
    
}

/*
* Used from evb interface. It returns:
* . 1000 if the handle is NULL; so the processId is not valid or not assiciated to this process
* . STILL_ACTIVE (259) the process is still active.
* . a different value indicates the proper exit value.
*/

int checkPid(long pid) {

    unsigned long use = pid;
    DWORD exitCode = 1000;
    wchar_t t[128];
    
    wsprintf(t, TEXT("%lu"), use);    
    
    HANDLE phandle = OpenProcess(0, FALSE, use);  
    
    if (phandle) {
        BOOL f = GetExitCodeProcess(phandle, &exitCode);
        CloseHandle(phandle);        
    } 
    
    return exitCode;
    
}

unsigned long startSyncPid(time_t timeout)
{
    if (!timeout)
        timeout=5*60; // default timeout 5 minutes

    unsigned long pid = startcmd(SYNCAPP, TEXT("once"));
    
    if (pid) {
        return pid;
    }
    else {
        return -5;
    }
}


/*      
 * Entry point to kickoff the synchronization process, both specifying the 
 * sources or not. The default is to make all the configured sources.
 *
 * @param path    : the install path of the SyncClient PIM application on 
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 */

DWORD WINAPI synchronize (wchar_t *errmsg, const wchar_t* path, const wchar_t** sources)
{
    wcscpy(errmsg, TEXT("Ok."));
    LOG.reset(); 
    int ret = 0, retMail = 0;
              
    if (sources)
        ret = synchronizeMultipleSources(path, sources);
    else
        ret = synchronization(errmsg, (wchar_t *)path);
    
    retMail = synchronizationMailOnly();
    if (retMail == -10 && ret == -10) {        
        LOG.debug("Mail not set to be synched");
        LOG.debug("No source to sync");
        return -10;
    }

    if (ret == -10) {
        ret = 0;
    }
    if (retMail == -10) {
        LOG.debug("Mail not set to be synched");
        retMail = 0;
    }
    if(ret != 0)
        return ret;
    else
        return retMail;
    
    return 0;
}

/*
    LOG_LEVEL_NONE  = 0
    LOG_LEVEL_INFO  = 1
    LOG_LEVEL_DEBUG = 2
*/

LogLevel getLOGLevel() {
    
    wchar_t level[64];
    memset(level, 0, 64*sizeof(wchar_t));
    LogLevel ret = LOG_LEVEL_NONE;
    long l = 0;

    getClientConfigurationInternal(NULL, PROPERTY_SPDM_LOG_LEVEL, level, FALSE);
    
    if (wcslen(level) != 0)
        l = wcstol(level, NULL, 0);
    
    ret = (LogLevel)l;

    return ret;    
}

/*
void testInsertMail(StringBuffer s) {
    MailServer2Client* mailServer2Client = new MailServer2Client();
    MailClientData m;
    m.setParent(TEXT("I"));
    m.emailData->parse(s.c_str());
    
    mailServer2Client->setFolderToSync(TEXT("I"));
    mailServer2Client->addMessage(&m);

}
*/




/*
 * NOTE: USED ONLY TEMPORANEALLY TO GET A MAIL PARAMETER THAT MUST
 * BE FIXED IN A PROPER WAY
 */
TCHAR* getMailMaxMsgSize() {
    HKEY key = NULL;    
    DWORD res;	
    long err = 0;
    TCHAR *p = TEXT("mailMaxMsgSize");
    ULONG dim = 0;
    TCHAR *buf = NULL;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        bsprintf(lastErrorMsg, T("Invalid context path: %s"), "Software/Funambol/SyncclientPIM/spds/sources/mails");
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            p,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
		if (dim > 0) {
            buf = new TCHAR[dim + 1];

			err = RegQueryValueEx(
					key,
					p,
					NULL,
					NULL,  // we currently support only strings
					(UCHAR*)buf,
					&dim
					);            
		}
    }
    //else MessageBox(NULL,  T("Error"), T("getConfigParameter"), MB_OK);
    
    if (!buf) 
        buf = wstrdup(TEXT(""));

finally:
  
    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}


/*
* return code -10: no source to sync
*/

DWORD synchronizationMailOnly () {
    
    wchar_t MAILS       [15];       
    int ret = 0;
    wcscpy(MAILS,         MAILS_NAME       );
    wchar_t t[12];
    wchar_t path [256];
    memset(path, 0, 256*sizeof(wchar_t));
    memset(t, 0, 12*sizeof(wchar_t));
    SyncSource** ssArray = new SyncSource*[2];
    TCHAR* msxmessagesize = getMailMaxMsgSize();
    unsigned long previousMaxMsgSize = 0;
    LogLevel ll = getLOGLevel();
    Sync4jClient& s4j = Sync4jClient::getInstance();
    int days = 0;
    int attachSize = 0;

    wchar_t* downloadAge = NULL;
    SourceFilter* filter = NULL;

    getClientConfigurationInternal (NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);
    
    // mail
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");   
    MailSyncSourceConfig sc;    
    sc = m.getMailSourceConfig(true);    
    MailSyncSource mailSource = MailSyncSource (MAILS); 
    DMTClientConfig config(APPLICATION_URI);

    if (strcmp(sc.getSync(), "none") == 0) {
        ret = -10;        
        goto finally;
    }
    // check the refresh from server 
    if (strcmp(sc.getSync(), "refresh-server") == 0 || strcmp(sc.getSync(), "refresh") == 0) {
        
        int result = 0;
        result = checkRefreshFromServer(OL_MAIL);
        
        if (result == -1) {
            ret = -10;        
            goto finally;
        }
    }
    
    config.read();    
    previousMaxMsgSize = config.getAccessConfig().getMaxMsgSize();
    config.getAccessConfig().setMaxMsgSize(wcstol(msxmessagesize, NULL, 10));    
    s4j.setDMConfig(config);    

    getFolderToSync(sc, t);    

    
    days = sc.getDownloadAge();
    downloadAge = getDayBefore(days);
    attachSize = sc.getAttachSize();
    //
    // create a default filter related on max message size. This permits to not loose any mail
    // even if it appears to large
    //
    if (attachSize == -1) {
        //reserved value for the remain part of syncml msg
        // 1024 because the filter multiply for 1024
        attachSize = (wcstol(msxmessagesize, NULL, 10) - 32768) / 1024;  
    }

    filter = 
        ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(), attachSize);        
    
    //
    //
    // create filter as default

    mailSource.setPath(path);
    mailSource.setFolderToSync(t);
    mailSource.setFilter(filter);
      
    ssArray[0] = &mailSource;
    ssArray[1] = NULL;                
        
    LOG.setLevel(ll);

    ret = s4j.sync(ssArray); 
    
    if (ret == 0) {
        config.getAccessConfig().setMaxMsgSize(previousMaxMsgSize);
        config.save();        
    }

    if (mailSource.getIsMailInOutbox() && ret == 0) {
        
        config.getAccessConfig().setMaxMsgSize(wcstol(msxmessagesize, NULL, 10));    
        s4j.setDMConfig(config);  
        ret = s4j.sync(ssArray);

        if (ret == 0) {
            config.getAccessConfig().setMaxMsgSize(previousMaxMsgSize);
            config.save();            
        }
    }
    
    if (mailSource.getIsMailInInbox()) {
        MessageBeep(MB_ICONASTERISK);
    }
    
    if (mailSource.getFailedSendMailInOutbox()) {
        //TimedMessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
        //    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, OUT_MESSAGE_TIMEOUT*1000);
        MessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
            MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);

    }

    Sync4jClient::dispose();
    LOG.debug(T("Mail Sync Ended."));

finally:
    if (msxmessagesize) { delete [] msxmessagesize; msxmessagesize = NULL;}
    return ret;
}


DWORD checkStartSync() {
    
    DWORD pid = 0;

    FILE *f = fopen("\\testlog.txt", "a+");
    // Try to create the mutex
    HANDLE hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );
    if(hMutex){
        if(GetLastError() == ERROR_ALREADY_EXISTS){
            pid = getProcessPid( TEXT("startsync.exe") );
            fprintf(f, "Found background sync. Pid is %ld", pid);
        }
        else{
            fputs("No bg sync", f);
        }
        CloseHandle( hMutex );
    }
    else {
       fputs("Error creating mutex in checkStartSync.", f);
    }
    return pid;

#if 0
    DWORD out = 0;
    PROCESSENTRY32 lppe; 
    BOOL next = TRUE;
    HANDLE hProcess = 0;
    DWORD ret = 0;

    HANDLE hSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); // 0 is however ignored
    
    if (hSnapshot == INVALID_HANDLE_VALUE) 
        return 0;
    
    lppe.dwSize = sizeof( PROCESSENTRY32 );

    if (!Process32First(hSnapshot, &lppe))
        return 0;

    if (wcsstr(lppe.szExeFile, TEXT("startsync.exe")) != NULL) {             
        ret = lppe.th32ProcessID;
        goto exit;
    }
    
    do {
        next = Process32Next(hSnapshot, &lppe);

        if (!next)
            break;
        
        if (wcsstr(lppe.szExeFile, TEXT("startsync.exe")) != NULL ) {                  
            ret = lppe.th32ProcessID; 
            goto exit;
        }

    } while(next);

exit:
    if (hSnapshot) 
        CloseToolhelp32Snapshot(hSnapshot);
    CloseHandle(hProcess);
    return ret;
#endif
}

#define ASK_REFRESH_MESSAGE TEXT("All the local %s will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_TIMEOUT 15

int checkRefreshFromServer(int type) {
    /*
    OL_CONTACTS     = 10,
    OL_CALENDAR     = 9,
    OL_TASK         = 13,
    OL_BRIEFCASE    = 600,
    OL_NOTES        = 601,
    OL_FAVORITES    = 602
    OL_MAIL         = 610
    */
    wchar_t message[256];
    
    wchar_t sourceNameW [128];       
    int ret = 0;
    message[0] = 0;  
    sourceNameW[0] = 0;

    if (type== OL_CONTACTS)
        wcscpy(sourceNameW, TEXT("contacts"));
    else if (type == OL_CALENDAR)
        wcscpy(sourceNameW, TEXT("calendars"));
    else if (type == OL_TASK)
        wcscpy(sourceNameW, TEXT("tasks"));
    else if (type == OL_NOTES)
        wcscpy(sourceNameW, TEXT("notes"));
    else if (type == OL_BRIEFCASE)
        wcscpy(sourceNameW, TEXT("files"));
    else if (type == OL_MAIL)
        wcscpy(sourceNameW, TEXT("mails"));
    
    wsprintf(message, ASK_REFRESH_MESSAGE, sourceNameW);
    
    int val = TimedMessageBox(NULL, message, TEXT("Alert"),
                         MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    if (val == IDYES) {
        LOG.debug("User said yes to delete local %S and refresh.", sourceNameW);            
    }
    else {
        LOG.debug("User said no to delete local %S and refresh." , sourceNameW);     
        ret = -1;
    }
   
    return ret;    
}
